import pytest
from src.content_parser.pdf_parser import PDFParser
from src.parsethisio import ResultFormat
import os

def test_pdf_parser():
    file_path = os.path.join(os.path.dirname(__file__), 'fixtures', 'text_data_meeting_notes.pdf')
    
    with open(file_path, "rb") as f:
        parser = PDFParser()
        text = parser.parse(f, result_format=ResultFormat.TXT)
        
    pypddf2ExpectedText= """YOUR
COMPANY 
MEETING
NAME
09/04
04
SEPTEMBER
20XX
/
4:30
PM
/
ROOM
436
ATTENDEES
Wendy
Writer,
Ronny
Reader,
Abby
Author
AGENDA
Last
Meeting
Follow-up
1.
Lorem
ipsum
dolor
sit
amet,
consectetuer
adipiscing
elit.
New
Business
●
Lorem
ipsum
dolor
sit
amet,
consectetuer
adipiscing
elit.
●
Suspendisse
scelerisque
mi
a
mi.
NOTES
●
Lorem
ipsum
dolor
sit
amet
consectetuer
adipiscing
elit.
●
Vestibulum
ante
ipsum
primis
elementum
,
libero
interdum
auctor
cursus,
sapien
enim
dictum
quam.
○
Phasellus
vehicula
nonummy
ACTION
ITEMS
1.
Lorem
ipsum
dolor
sit
amet
consectetuer
adipiscing
elit.
NEXT
W EEK’S
AGENDA
Lorem
ipsum
dolor
sit
amet,
consectetuer
adipiscing
elit.
"""

    assert text == pypddf2ExpectedText



import os
from src.content_parser.archive_parser import ArchiveParser
from src.utils import ResultFormat
import filetype

def test_archive_parser():
    file_path = os.path.join(os.path.dirname(__file__), 'fixtures', 'test_data_archive.zip')
    
    with open(file_path, "rb") as f:
        content = f.read()
        mime = filetype.guess(content).mime
        assert mime == "application/zip"
        
        parser = ArchiveParser()
        # Test TXT format
        text = parser.parse(content, result_format=ResultFormat.TXT)
        assert isinstance(text, str)
        assert len(text) > 0
        
        # Test MD format
        md_text = parser.parse(content, result_format=ResultFormat.MD)
        assert isinstance(md_text, str)
        assert len(md_text) > 0
        assert "# Archive Contents" in md_text



from .base_parser import BaseParser
from markitdown import MarkItDown
from io import BytesIO
from ..utils import ResultFormat
import tempfile
import filetype

class OfficeParser(BaseParser):
    def __init__(self):
        self.converter = MarkItDown()

    @property
    def supported_mimetypes(self) -> list:
        return [
            "application/vnd.openxmlformats-officedocument.wordprocessingml.document",  # docx
            "application/vnd.openxmlformats-officedocument.presentationml.presentation",  # pptx
            "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",  # xlsx
            "application/vnd.ms-excel",  # xls
        ]

    def parse(self, file_content, result_format: ResultFormat) -> str:
        # Handle both bytes and file-like objects
        if hasattr(file_content, 'read'):
            content = file_content.read()
        else:
            content = file_content

        # Detect file extension based on mime type from magic/filetype
        extension = '.docx'  # default
        mime_type = filetype.guess(content).mime
        if 'presentation' in mime_type:
            extension = '.pptx'
        elif 'sheet' in mime_type or 'excel' in mime_type:
            extension = '.xlsx' if 'openxmlformats' in mime_type else '.xls'

        # MarkItDown expects a file path, so we need to write to a temp file
        with tempfile.NamedTemporaryFile(suffix=extension) as temp_file:
            temp_file.write(content)
            temp_file.flush()
            result = self.converter.convert(temp_file.name)
            text = result.text_content
            if result_format == ResultFormat.MD:
                # Convert first line to markdown header
                lines = text.split('\n')
                if lines:
                    lines[0] = f"# {lines[0]}"
                text = '\n'.join(lines)
            return text



from .base_parser import BaseParser
from PyPDF2 import PdfReader
from markitdown import MarkItDown
from io import BytesIO
from ..utils import ResultFormat
import tempfile

class PDFParser(BaseParser):
    def __init__(self):
        self.converter = MarkItDown()

    @property
    def supported_mimetypes(self) -> list:
        return [
            "application/pdf"
        ]

    def parse(self, file_content, result_format: ResultFormat) -> str:
        if result_format == ResultFormat.TXT:
            # Use PyPDF2 for backward compatibility
            if isinstance(file_content, bytes):
                file_content = BytesIO(file_content)
            reader = PdfReader(file_content)
            text = ""
            for page_num in range(len(reader.pages)):
                page = reader.pages[page_num]
                text += page.extract_text()
            return text
        else:
            # Use Markitdown for MD format
            if hasattr(file_content, 'read'):
                content = file_content.read()
            else:
                content = file_content
            with tempfile.NamedTemporaryFile(suffix='.pdf') as temp_file:
                temp_file.write(content)
                temp_file.flush()
                result = self.converter.convert(temp_file.name)
                return result.text_content



import os
import mimetypes
from src.content_parser.data_parser import DataParser
from src.utils import ResultFormat

def test_csv_parser():
    file_path = os.path.join(os.path.dirname(__file__), 'fixtures', 'test_data_table.csv')
    
    with open(file_path, "rb") as f:
        content = f.read()
        mime = mimetypes.guess_type(file_path)[0]
        assert mime == "text/csv"
        
        parser = DataParser()
        text = parser.parse(content, result_format=ResultFormat.TXT)
        assert isinstance(text, str)
        assert len(text) > 0
        assert "名前,年齢,住所" in text  # Check CSV header (Japanese)

def test_json_parser():
    file_path = os.path.join(os.path.dirname(__file__), 'fixtures', 'test_data_config.json')
    
    with open(file_path, "rb") as f:
        content = f.read()
        mime = mimetypes.guess_type(file_path)[0]
        assert mime == "application/json"
        
        parser = DataParser()
        # Test TXT format
        text = parser.parse(content, result_format=ResultFormat.TXT)
        assert isinstance(text, str)
        assert len(text) > 0
        assert '"key1": "string_value"' in text

        # Test MD format
        md_text = parser.parse(content, result_format=ResultFormat.MD)
        assert isinstance(md_text, str)
        assert len(md_text) > 0
        assert "```json" in md_text

def test_xml_parser():
    file_path = os.path.join(os.path.dirname(__file__), 'fixtures', 'test_data_feed.xml')
    
    with open(file_path, "rb") as f:
        content = f.read()
        mime = mimetypes.guess_type(file_path)[0]
        assert mime == "application/xml"
        
        parser = DataParser()
        text = parser.parse(content, result_format=ResultFormat.TXT)
        assert isinstance(text, str)
        assert len(text) > 0
        assert "The Official Microsoft Blog" in text



import pytest
from unittest.mock import patch, MagicMock
from src.content_parser.audio_parser import AudioParser
from src.parsethisio import ResultFormat
import os
# src/content_parser/test_audio_parser.py

audio_test_file_path = os.path.join(os.path.dirname(__file__), 'fixtures', 'test_data_ttsmaker-test-generated-file.mp3')
audio_test_file_transkript = "Welcome to Eleven Degrees, your gateway to the cutting edge of innovation and the world of technology. I’m your host, Sam, and each week, we explore the latest trends, breakthroughs, and the people shaping the future of technology."

@pytest.fixture
def mock_openai():
    with patch('src.content_parser.audio_parser.OpenAI') as MockOpenAI:
        mock_client = MockOpenAI.return_value
        mock_client.audio.transcriptions.create.return_value = MagicMock(
            text=audio_test_file_transkript
        )
        yield mock_client

def test_audio_parser(mock_openai):
    parser = AudioParser()
    with open(audio_test_file_path, "rb") as f:
        result = parser.parse(f.read(), result_format=ResultFormat.TXT)

    assert result == audio_test_file_transkript
    mock_openai.audio.transcriptions.create.assert_called_once()

def test_audio_parser_markdown_result_equals_text_result(mock_openai):
    parser = AudioParser()
    with open(audio_test_file_path, "rb") as f:
        result = parser.parse(f.read(), result_format=ResultFormat.MD)

    assert result == audio_test_file_transkript
    mock_openai.audio.transcriptions.create.assert_called_once()




import pytest
from unittest.mock import patch, MagicMock
from bs4 import BeautifulSoup, Tag
from youtube_transcript_api import YouTubeTranscriptApi
from src.content_parser.helpers.youtube_transcript_helper import YouTubeTranscriptHelper
from src.exceptions import RegexResultError, RemoteRequestError, NotFoundError

@pytest.fixture
def sample_urls():
    return {
        "valid": "https://www.youtube.com/watch?v=dQw4w9WgXcQ",
        "invalid": "https://youtube.com/invalid",
        "shorts": "https://youtube.com/shorts/dQw4w9WgXcQ",
        "embed": "https://www.youtube.com/embed/dQw4w9WgXcQ"
    }

@pytest.fixture
def mock_requests():
    with patch('requests.get') as mock_get:
        yield mock_get

@pytest.fixture
def mock_transcript_api():
    with patch('src.content_parser.helpers.youtube_transcript_helper.YouTubeTranscriptApi') as mock_api:
        mock_list = MagicMock()
        mock_api.list_transcripts = mock_list
        yield mock_api

def test_extract_video_id_valid(sample_urls):
    """Test that valid YouTube URL extracts correct video ID."""
    helper = YouTubeTranscriptHelper()
    video_id = helper.extract_video_id(sample_urls["valid"])
    assert video_id == "dQw4w9WgXcQ"

def test_extract_video_id_shorts(sample_urls):
    """Test that YouTube shorts URL extracts correct video ID."""
    helper = YouTubeTranscriptHelper()
    video_id = helper.extract_video_id(sample_urls["shorts"])
    assert video_id == "dQw4w9WgXcQ"

def test_extract_video_id_embed(sample_urls):
    """Test that YouTube embed URL extracts correct video ID."""
    helper = YouTubeTranscriptHelper()
    video_id = helper.extract_video_id(sample_urls["embed"])
    assert video_id == "dQw4w9WgXcQ"

def test_extract_video_id_invalid(sample_urls):
    """Test that invalid YouTube URL raises RegexResultError."""
    helper = YouTubeTranscriptHelper()
    with pytest.raises(RegexResultError):
        helper.extract_video_id(sample_urls["invalid"])

def test_get_video_title_success(mock_requests):
    """Test successful video title extraction."""
    mock_requests.return_value.text = '<meta property="og:title" content="Test Video">'
    helper = YouTubeTranscriptHelper()
    title = helper.get_video_title("dQw4w9WgXcQ")
    assert title == "Test Video"

def test_get_video_title_network_error(mock_requests):
    """Test network error handling in title extraction."""
    mock_requests.side_effect = Exception("Network error")
    helper = YouTubeTranscriptHelper()
    with pytest.raises(RemoteRequestError):
        helper.get_video_title("dQw4w9WgXcQ")

def test_get_video_title_missing_meta(mock_requests):
    """Test handling of missing meta tag in title extraction."""
    mock_requests.return_value.text = '<html><body>No meta tag</body></html>'
    helper = YouTubeTranscriptHelper()
    with pytest.raises(RemoteRequestError, match="Failed to fetch video title for dQw4w9WgXcQ"):
        helper.get_video_title("dQw4w9WgXcQ")

def test_get_best_transcript_manual(mock_transcript_api):
    """Test successful manual transcript retrieval."""
    mock_transcript = MagicMock()
    mock_transcript.is_generated = False
    mock_transcript.language_code = "en"
    mock_transcript.fetch.return_value = [{"text": "Test", "start": 0, "duration": 1}]
    
    mock_transcript_api.list_transcripts.return_value = [mock_transcript]
    
    helper = YouTubeTranscriptHelper()
    transcript = helper.get_best_transcript("dQw4w9WgXcQ")
    assert transcript == [{"text": "Test", "start": 0, "duration": 1}]

def test_get_best_transcript_auto_generated(mock_transcript_api):
    """Test auto-generated transcript fallback."""
    mock_transcript = MagicMock()
    mock_transcript.is_generated = True
    mock_transcript.language_code = "en"
    mock_transcript.fetch.return_value = [{"text": "Auto", "start": 0, "duration": 1}]
    
    mock_transcript_api.list_transcripts.return_value = [mock_transcript]
    
    helper = YouTubeTranscriptHelper()
    transcript = helper.get_best_transcript("dQw4w9WgXcQ")
    assert transcript == [{"text": "Auto", "start": 0, "duration": 1}]

def test_get_best_transcript_translation(mock_transcript_api):
    """Test translation fallback when no preferred language available."""
    mock_transcript = MagicMock()
    mock_transcript.is_translatable = True
    mock_transcript.translate.return_value.fetch.return_value = [{"text": "Translated", "start": 0, "duration": 1}]
    
    mock_transcript_api.list_transcripts.return_value = [mock_transcript]
    
    helper = YouTubeTranscriptHelper()
    transcript = helper.get_best_transcript("dQw4w9WgXcQ")
    assert transcript == [{"text": "Translated", "start": 0, "duration": 1}]

def test_get_best_transcript_not_found(mock_transcript_api):
    """Test error handling when no transcript is available."""
    mock_transcript_api.list_transcripts.return_value = []
    
    helper = YouTubeTranscriptHelper()
    with pytest.raises(RemoteRequestError, match="Failed to get transcript for video dQw4w9WgXcQ: No suitable transcript found"):
        helper.get_best_transcript("dQw4w9WgXcQ")

def test_clean_transcript_default_phrases():
    """Test removal of default phrases from transcript."""
    helper = YouTubeTranscriptHelper()
    transcript = [
        {"text": "[music] Hello", "start": 0, "duration": 1},
        {"text": "World [applause]", "start": 1, "duration": 1},
        {"text": "[laughter] Test", "start": 2, "duration": 1}
    ]
    cleaned = helper.clean_transcript(transcript)
    assert cleaned == "Hello World Test"

def test_clean_transcript_custom_phrases():
    """Test removal of custom phrases from transcript."""
    helper = YouTubeTranscriptHelper()
    transcript = [
        {"text": "[intro] Hello", "start": 0, "duration": 1},
        {"text": "World [outro]", "start": 1, "duration": 1}
    ]
    cleaned = helper.clean_transcript(transcript, remove_phrases=["[intro]", "[outro]"])
    assert cleaned == "Hello World"

def test_clean_transcript_invalid_entries():
    """Test handling of invalid transcript entries."""
    helper = YouTubeTranscriptHelper()
    transcript = [
        {"text": "Valid"},
        {"wrong_key": "Invalid"},
        {"text": 123},
        {"text": "Also Valid"}
    ]
    cleaned = helper.clean_transcript(transcript)
    assert cleaned == "Valid Also Valid"



import unittest
from unittest.mock import patch, MagicMock
import src.parsethisio
from src.content_parser.audio_parser import AudioParser
from src.content_parser.image_parser import ImageParser
from src.content_parser.pdf_parser import PDFParser
from src.content_parser.text_parser import TextParser
from io import BytesIO
import os

class TestAutomaticParserSelection(unittest.TestCase):
    def test_get_image_parser(self):
        file_path = os.path.join(os.path.dirname(__file__), 'fixtures', 'test_data_diagram.png')
    
        with open(file_path, "rb") as f:
            # Call the function
            parser = src.parsethisio.get_parser(f.read())

            #assert parser is of class ImageParser
            assert isinstance(parser, ImageParser)

    def test_get_pdf_parser(self):
        file_path = os.path.join(os.path.dirname(__file__), 'fixtures', 'text_data_meeting_notes.pdf')
    
        with open(file_path, "rb") as f:
            # Call the function
            parser = src.parsethisio.get_parser(f.read())

            #assert parser is of class ImageParser
            assert isinstance(parser, PDFParser)

    def test_get_audio_parser(self):
        file_path = os.path.join(os.path.dirname(__file__), 'fixtures', 'test_data_ttsmaker-test-generated-file.mp3')
    
        with open(file_path, "rb") as f:
            # Call the function
            parser = src.parsethisio.get_parser(f.read())

            #assert parser is of class ImageParser
            assert isinstance(parser, AudioParser)

    def test_get_text_parser_by_url(self):
        # Call the function
        parser = src.parsethisio.get_parser("http://jdde.de", mime_type="text/plain")

        #assert parser is of class ImageParser
        assert isinstance(parser, TextParser)

    def test_get_text_parser_by_url(self):
        # Call the function
        parser = src.parsethisio.get_parser("This is a normal, awesome text", mime_type="text/plain")

        #assert parser is of class ImageParser
        assert isinstance(parser, TextParser)


if __name__ == '__main__':
    unittest.main()